unit DataMod;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Db, DBTables, inifiles;


type
  TDM = class(TDataModule)
    dbPPdata: TDatabase;
    procedure DBSaccLogin(Database: TDatabase; LoginParams: TStrings);
    procedure DMCreate(Sender: TObject);
    function EncodeString(const ClientCode: string): string;
    function DecodeString(const Key: string): string;
    procedure SetDatabaseSet(const TestSet: boolean);
  private
    { Private declarations }
    function StringCode(const CodedWord: String): String;
    function EncryptIt(const TheString: string): string;
    procedure OpenMainServerDB;
  public
    { Public declarations }
  end;

var
  DM: TDM;

implementation

uses
  ConstantValuesSys, LoginFrm;
  
{$R *.DFM}

function TDM.StringCode(const CodedWord: String) : String;
var
  CodedWordLength : Integer;
  CodedWordCharNumber : Integer;
  AsciiValue : Integer;
  DecodedWord : string;
begin
  CodedWordLength := Length( CodedWord );
  DecodedWord := '';
  for CodedWordCharNumber := 1 to CodedWordLength do
    begin
    // Get the ASCII value of the next character in the passed string.
    AsciiValue := Ord( CodedWord[ CodedWordCharNumber ] );
    // XOR the ASCII code.
    AsciiValue := (AsciiValue xor (-1));
    // Convert the ASCII code to a character, add to the result string.
    DecodedWord := DecodedWord + Chr( AsciiValue );
    end;
  Result := DecodedWord;
end;


function TDM.EncodeString(const ClientCode: string): string;
var
  CryptBuf : String;
  HexBuf : String;
  HexChar : String;
  i : Integer;
begin
  CryptBuf := EncryptIt(ClientCode);
  for i := 1 To Length(CryptBuf) do begin
    HexChar := IntToHex(Ord(CryptBuf[ i ]),2);
    if (Length(HexChar) = 1) then begin
       HexChar := '0' + HexChar;
    end;
    HexBuf := HexBuf + HexChar;
  end;
  Result := HexBuf;
end;  // EncodeString.

function TDM.DecodeString(const Key: string): string;
var
  Buf : String;
  i : integer;
begin
  i := 1;
  while i < Length(Key) do begin
    Buf := Buf + Chr(StrToInt('$' + Copy(Key, i, 2)));       // Hex to Char.
    Inc(i, 2);  // effectively steps by 2.
  end;
  Result := EncryptIt(Buf);
end;  // DecodeString.

function TDM.EncryptIt(const TheString: string): string;
const
  SIMPLE_KEY = '~1m2D3b4C5o6N7s8U9l0T1i2N3g4~';
var
  i : integer;
  Key : string;
  Jumble : string;
begin

  //Make the key longer than 'TheString'
  while (Length(Key) < Length(TheString)) do
    Key := Key + SIMPLE_KEY;

  for i := 1 To Length(TheString) do begin
    Jumble := Jumble + Chr(Ord(TheString[ i ]) xor Ord(Key[ i ]));
  end;

  EncryptIt := Jumble;

end;  // EncryptIt.


procedure TDM.DBSaccLogin(Database: TDatabase;
  LoginParams: TStrings);
var
  KeyString : string;
begin
  KeyString := Chr(147)+Chr(141)+Chr(145)+Chr(146)+
                       Chr(143)+Chr(135);
  LoginParams.Values['USERNAME'] := 'ibmsys';
  KeyString := StringCode(KEYSTRING);
  LoginParams.Values['PASSWORD'] := KeyString;          
end;


procedure TDM.OpenMainServerDB;
var
  NewPPIniFile : TIniFile;
  IniFileName : string;
  LoginParams : TStringList;
  sDSN : string;
  sUserName : string;
  sPassword : string;
begin
  LoginParams := TStringList.Create;
  IniFileName := INIFILEDIRECTORY+ChangeFileExt(ExtractFileName(ParamStr(0)),'.ini');
  NewPPIniFile := TIniFile.Create(IniFileName);
  // dbPPdata - SQLPP (most common) database.
  with NewPPIniFile do begin
    sDSN := ReadString('DBconnection','PPDSN', '');
    sUserName := DecodeString(ReadString('DBconnection','ur', ''));
    sPassword := DecodeString(ReadString('DBconnection','pd', ''));
  end;
  // True in the following = Startup call only.
  if (Length(sDSN) = 0) or (Length(sUserName) = 0)
    or (Length(sPassword) = 0) then begin
    if (Length(sDSN) = 0) then sDSN := dbPPdata.AliasName;
    GetLoginParams('PreProcessor',True,sDSN,LoginParams);
    sUserName := LoginParams.Values['USER NAME'];
    sPassword := LoginParams.Values['PASSWORD'];
    with NewPPIniFile do begin
      if (Length(sDSN)>0) then
        WriteString('DBconnection','PPDSN',sDSN);
      if (Length(sUserName)>0) then
        WriteString('DBconnection','ur',EncodeString(sUserName));
      if (Length(sPassword)>0) then
        WriteString('DBconnection','pd',EncodeString(sPassword));
    end;  // with NewPPIniFile do begin.
  end;  // One or more parameters missing.
  if (Length(sUserName) > 0) and (Length(sPassword) > 0) then begin
    LoginParams.Add('USER NAME='+sUserName);
    LoginParams.Add('PASSWORD='+sPassword);
    if dbPPdata.Connected then dbPPdata.Connected := False;
    dbPPdata.AliasName := sDSN;
    dbPPdata.Params := LoginParams;
    dbPPdata.LoginPrompt := False;
  end;  // One or more parameters missing.
  dbPPdata.Session.NetFileDir := INIFILEDIRECTORY;
//  dbPPdata.Session.PrivateDir := INIFILEDIRECTORY;
  dbPPdata.Connected := True;
  // Done - Logged into the Main database.
  LoginParams.Free;
  NewPPIniFile.UpdateFile;
  NewPPIniFile.Free;
end;  // OpenMainServerDB.


procedure TDM.SetDatabaseSet(const TestSet: boolean);
var
  NewPPIniFile : TIniFile;
  IniFileName : string;
  LoginParams : TStringList;
  sDSN : string;
  sUserName : string;
  sPassword : string;
  lTest : boolean;
  sDBConnection : string;
begin
  lTest := TestSet;
  LoginParams := TStringList.Create;
  IniFileName := INIFILEDIRECTORY+ChangeFileExt(ExtractFileName(ParamStr(0)),'.ini');
  NewPPIniFile := TIniFile.Create(IniFileName);
  if lTest then
    sDBConnection := 'TestCfg'
  else
    sDBConnection := 'MainCfg';
  with NewPPIniFile do begin
    sDSN := DecodeString(ReadString(sDBConnection,'id1', ''));
    sUserName := DecodeString(ReadString(sDBConnection,'id2', ''));
    sPassword := DecodeString(ReadString(sDBConnection,'id3', ''));
  end;
  // True in the following = Startup call only.
  if (Length(sDSN) = 0) or (Length(sUserName) = 0)
    or (Length(sPassword) = 0) then begin
    if (Length(sDSN) = 0) then begin
      sDSN := dbPPdata.AliasName;
      if lTest then sDSN := sDSN + '_TESTDB';
    end;
    GetLoginParams('PreProcessor',True,sDSN,LoginParams);
    sUserName := LoginParams.Values['USER NAME'];
    sPassword := LoginParams.Values['PASSWORD'];
  end;
  with NewPPIniFile do begin
    if (Length(sDSN)>0) then begin
      WriteString('DBconnection','PPDSN',sDSN);
      WriteString(sDBConnection,'id1',EncodeString(sDSN));
    end;
    if (Length(sUserName)>0) then begin
      WriteString('DBconnection','ur',EncodeString(sUserName));
      WriteString(sDBConnection,'id2',EncodeString(sUserName));
    end;
    if (Length(sPassword)>0) then begin
      WriteString('DBconnection','pd',EncodeString(sPassword));
      WriteString(sDBConnection,'id3',EncodeString(sPassword));
    end;
  end;  // with NewPPIniFile do begin.
  OpenMainServerDB;
  // Done - Passwords are set correctly.
  LoginParams.Free;
  NewPPIniFile.UpdateFile;
  NewPPIniFile.Free;
end;  //  SetDatabaseSet.

procedure TDM.DMCreate(Sender: TObject);
begin
  OpenMainServerDB;
end;

end.
