unit UserBilling;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Grids, StdCtrls, Mask, DBCtrls, Db, DBTables, DBGrids;

type
  TUserBillingForm = class(TForm)
    dbeCustno: TDBEdit;
    lblCustno: TLabel;
    lblBillAddr: TLabel;
    cbbLookUpAddr: TDBLookupComboBox;
    txtAddr01: TLabel;
    txtAddr02: TLabel;
    txtAddr03: TLabel;
    txtAddr04: TLabel;
    txtAddr05: TLabel;
    txtAddr06: TLabel;
    dbtRegion: TDBText;
    dbtUser: TDBText;
    dbtUserloc: TDBText;
    dbtDept: TDBText;
    dbtGroup: TDBText;
    lblBillingDetails: TLabel;
    lblCustnoDetails: TLabel;
    btnAddBillAddr: TButton;
    btnDeleteAddr: TButton;
    btnClose: TButton;
    lblGridAddr: TLabel;
    dbgLinks: TDBGrid;
    DBCustomers: TDatabase;
    tblLinks: TTable;
    DSLinks: TDataSource;
    sesUserCustomer: TSession;
    txtCustomerCode: TLabel;
    txtShipToCode: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure cbbLookUpAddrClick(Sender: TObject);
    procedure UpdateAddressInfo;
    procedure cbbLookUpAddrKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure cbbLookUpAddrKeyPress(Sender: TObject; var Key: Char);
    procedure cbbLookUpAddrCloseUp(Sender: TObject);
    procedure btnCloseClick(Sender: TObject);
    procedure btnAddBillAddrClick(Sender: TObject);
    procedure btnDeleteAddrClick(Sender: TObject);
    procedure dbgLinksDblClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  UserBillingForm: TUserBillingForm;

implementation

uses DataModUser;

var
  AddrIndex : String;        // Saves current index of tblBillAddr.
  AddrMs    : TDataSource;   // Saves current master datasource of tblBillAddr.
  AddrMf    : string;        // Saves current master fields of tblBillAddr.
  lBillAddrActive : Boolean; // Stores the open status of the tblBillAddr table.

{$R *.DFM}

procedure TUserBillingForm.FormCreate(Sender: TObject);
begin
  // Undo the master / slave controls between tblUser and tblBillAddr.
  AddrIndex   := DMUser.tblBillAddr.IndexName;
  DMUser.tblBillAddr.IndexName := 'BillAddr_CustomerCode';
  AddrMs      := DMUser.tblBillAddr.MasterSource;
  DMUser.tblBillAddr.MasterSource := Nil;
  AddrMf      := DMUser.tblBillAddr.MasterFields;
  DMUser.tblBillAddr.MasterFields := '';

  // Open the table if not already done.
  lBillAddrActive := DMUser.tblBillAddr.Active;
  if not lBillAddrActive then
    begin
    DMUser.tblBillAddr.Open;
    end;

  // Now display the linking parameters in the string grid.
  tblLinks.Filter := 'custno=''' + DMUser.tblUser['custno'] + '''';
  tblLinks.Filtered := True;
  tblLinks.Open;
  tblLinks.First;

  // Now move to the required record value.
  if tblLinks.Eof or (tblLinks.FieldByName('CustomerCode').AsString = '') then
    // No link is in place, or no link exists.
    begin
    DMUser.tblBillAddr.First;
    txtAddr01.Caption := '';
    txtAddr02.Caption := '';
    txtAddr03.Caption := '';
    txtAddr04.Caption := '';
    txtAddr05.Caption := '';
    txtAddr06.Caption := '';
    txtCustomerCode.Caption := '';
    txtShipToCode.Caption := '';
    end
  else
    begin
    DMUser.tblBillAddr.FindKey(
      [tblLinks['CustomerCode'],tblLinks['ShipToCode']]);
    // Now toggle the LookUpComboBox to force an update.
    cbbLookUpAddr.KeyValue :=
      DMUser.tblBillAddr.FieldByName('ShortName').AsString;
    // Update the adress information by calling the default procedure.
    cbbLookUpAddrClick(Nil);
    end;

  DMUser.tblBillAddr.IndexName := 'BillAddr_ShortName';
end;


procedure TUserBillingForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  // Remove the filtering applied.
  tblLinks.Filtered := False;
  tblLinks.Filter := '';
  tblLinks.Close;
  DMUser.tblBillAddr.IndexName    := AddrIndex;
  DMUser.tblBillAddr.MasterSource := AddrMs;
  DMUser.tblBillAddr.MasterFields := AddrMf;

  if not lBillAddrActive then
    begin
    DMUser.tblBillAddr.Close;
    end;
end;


procedure TUserBillingForm.UpdateAddressInfo;
begin
  txtAddr01.Caption := DMUser.tblBillAddr['Addr01'];
  txtAddr02.Caption := DMUser.tblBillAddr['Addr02'];
  txtAddr03.Caption := DMUser.tblBillAddr['Addr03'];
  txtAddr04.Caption := DMUser.tblBillAddr['Addr04'];
  txtAddr05.Caption := DMUser.tblBillAddr['Addr05'];
  txtAddr06.Caption := DMUser.tblBillAddr['Addr06'];
  txtCustomerCode.Caption := DMUser.tblBillAddr['CustomerCode'];
  txtShipToCode.Caption := DMUser.tblBillAddr['ShipToCode'];
end;


procedure TUserBillingForm.cbbLookUpAddrClick(Sender: TObject);
begin
  UpdateAddressInfo;
end;


procedure TUserBillingForm.cbbLookUpAddrKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  UpdateAddressInfo;
end;


procedure TUserBillingForm.cbbLookUpAddrKeyPress(Sender: TObject;
  var Key: Char);
begin
  UpdateAddressInfo;
end;


procedure TUserBillingForm.cbbLookUpAddrCloseUp(Sender: TObject);
begin
  UpdateAddressInfo;
end;


procedure TUserBillingForm.btnCloseClick(Sender: TObject);
var
  lEndRoutine : Boolean;
  nRecords : Longint;

  nThisIntRate : Longint;

  nRentPortion : Longint;
  nCallPortion : Longint;
  nOtherPortion: Longint;
begin
  lEndRoutine := False;
  // Check values are valid.
  if tblLinks.Bof and tblLinks.Eof then
    // Add an unlinked record.
    begin
    tblLinks.Append;
    tblLinks['custno'] := DMUser.tblUser['custno'];
    tblLinks['SharedCosts'] := False;
    tblLinks.Post;
    lEndRoutine := True;
    end
  else
    begin
    nRecords := tblLinks.RecordCount;
    // Move to the required link.
    tblLinks.First;
    if (nRecords = 1) then
      // Reset the proportions of costing to 100%.
      begin
        tblLinks.Edit;
        tblLinks['RentRate']  := 100;
        tblLinks['CallRate']  := 100;
        tblLinks['OtherRate'] := 100;
        tblLinks['SharedCosts'] := False;
        tblLinks.Post;
        lEndRoutine := True;
      end
    else if (nRecords > 1) then
      // Ensure that the total of the proportioned rates is 100.
      begin
      // Initialize running totals.
      nRentPortion  := 0;
      nCallPortion  := 0;
      nOtherPortion := 0;
      while not tblLinks.Eof and
        (tblLinks['custno']=DMUser.tblUser['custno']) do
        begin
        tblLinks.Edit;
        // First, standardise the resolution of the divisions - 1 decimal place
        // and then sum the rounded values.
        if (tblLinks['rentrate']=Null) then
          // Reinstate a valid number.
          begin
          tblLinks['rentrate'] := 0.0;
          end;
        nThisIntRate := Round(10*tblLinks['rentrate']);
        tblLinks['rentrate'] := nThisIntRate / 10;
        nRentPortion  := nRentPortion  + nThisIntRate;
        // Repeat for the Call Rate.
        if (tblLinks['callrate']=Null) then
          // Reinstate a valid number.
          begin
          tblLinks['callrate'] := 0.0;
          end;
        nThisIntRate := Round(10*tblLinks['callrate']);
        tblLinks['callrate'] := nThisIntRate / 10;
        nCallPortion  := nCallPortion  + nThisIntRate;
        // Repeat for the Other Rate.
        if (tblLinks['otherrate']=Null) then
          // Reinstate a valid number.
          begin
          tblLinks['otherrate'] := 0.0;
          end;
        nThisIntRate := Round(10*tblLinks['otherrate']);
        tblLinks['otherrate'] := nThisIntRate / 10;
        nOtherPortion := nOtherPortion + nThisIntRate;
        tblLinks['SharedCosts'] := True;
        tblLinks.Post;
        tblLinks.Next;    // Also Posts the changes made.
        end;
      if (nRentPortion = 1000) and
        (nCallPortion = 1000) and
        (nOtherPortion = 1000) then
        begin
        lEndRoutine := True;
        end
      else
        begin
        MessageDlg('Proportioning of the bill does not total 100%.' +
          Chr(13)+Chr(10)+'Please correct the values.', mtWarning, [mbOK],0);
        end;    // if (nRentPortion = 1000) and..else.
      end
    else
      // Something is wrong.
      begin
      //
      end;
    end;
  if lEndRoutine then
    begin
    Close;
    end;
end;


procedure TUserBillingForm.btnAddBillAddrClick(Sender: TObject);
var
  lAlreadyUsed : Boolean;
  lHaveBlankNum : Boolean;
  BM_1 : TBookMarkStr;
  BM_2 : TBookMarkStr;
begin
  // Check that a billing address has been selected before proceeding.
  if (cbbLookUpAddr.Text <> '') then
    begin
    // Check that this address is not already included in the links.

    lAlreadyUsed := False;
    lHaveBlankNum := False;
    BM_1 := tblLinks.BookMark;

    // Note: tblLinks is filtered - for NOW!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    if not (tblLinks.Bof and tblLinks.Eof) then
      begin
      tblLinks.First;
      end;
    while not tblLinks.Eof do
      begin
      if (tblLinks.FieldByName('CustomerCode').AsString =
        DMUser.tblBillAddr['CustomerCode']) and
        (tblLinks.FieldByName('ShipToCode').AsString =
        DMUser.tblBillAddr['ShipToCode']) then
        // Add an new entry.
        begin
        lAlreadyUsed := True;
        break;
        end
      else if tblLinks.FieldByName('CustomerCode').AsString = '' then
        // Replace the blank entry.
        begin
        lHaveBlankNum := True;
        BM_2 := tblLinks.BookMark;
        end;
      tblLinks.Next;
      end;
    tblLinks.BookMark := BM_1;

    if not lAlreadyUsed then
      // This address has not been included, so add it.
      begin
      DMUser.tblUser.Edit;
      DMUser.tblUser['CustomerCode'] := DMUser.tblBillAddr['CustomerCode'];
      DMUser.tblUser.Post;

      if lHaveBlankNum then
        begin
        tblLinks.Edit;
        end
      else
        begin
        tblLinks.Append;
        end;
      tblLinks['CustomerCode']:= DMUser.tblBillAddr['CustomerCode'];
      tblLinks['ShipToCode'] := DMUser.tblBillAddr['ShipToCode'];
      tblLinks['custno'] := DMUser.tblUser['custno'];
      tblLinks['CustomerName'] :=
        Copy(DMUser.tblBillAddr['Addr01'],1,10);
      tblLinks['Status'] := DMUser.tblBillAddr['CustomerStatus'];
      tblLinks['SharedCosts'] := True;
      tblLinks.Post;
      end;    // if not lAlReadyUsed then.
    end;    // if (cbbLookUpAddr.Text <> '') then.
end;


procedure TUserBillingForm.btnDeleteAddrClick(Sender: TObject);
begin
  // Delete the current link.
  if not (tblLinks.Bof and tblLinks.Eof) then
    // Delete this link and refresh the data.
    begin
    tblLinks.Delete;
    end;
  // Now update the CustomerCode in the User table.
  tblLinks.First;
  DMUser.tblUser.Edit;
  if tblLinks.Bof and tblLinks.Eof then
    // Delete this link and refresh the data.
    begin
    DMUser.tblUser['CustomerCode'] := '';
    end
  else
    begin
    DMUser.tblUser['CustomerCode'] := tblLinks['CustomerCode'];
    end;
  DMUser.tblUser.Post;
end;


procedure TUserBillingForm.dbgLinksDblClick(Sender: TObject);
var
  IndexName : string;
begin
  if not (tblLinks.Bof and tblLinks.Eof) and (tblLinks['CustomerCode']<>'') then
    begin
    IndexName := DMUser.tblBillAddr.IndexName;
    DMUser.tblBillAddr.IndexName := 'BillAddr_CustomerCode';
    DMUser.tblBillAddr.FindKey(
      [tblLinks['CustomerCode'],tblLinks['ShipToCode']]);
    DMUser.tblBillAddr.IndexName := IndexName;
    UpdateAddressInfo;
    end;
end;

end.
